@@ -1,3 +1,7 @@
+"""os-urlpattern.
+
+Unsupervised URLs clustering, generate and match URL pattern.
+"""
 import sys
 __all__ = ['__version__', 'version_info']
 
@@ -7,8 +11,7 @@ version_info = tuple(int(v) if v.isdigit() else v
                      for v in __version__.split('.'))
 
 if sys.version_info < (2, 7):
-    print("os-urlpattern %s requires Python 2.7" % __version__)
-    sys.exit(1)
+    sys.exit("os-urlpattern %s requires Python 2.7" % __version__)
 
 del pkgutil
 del sys
