@@ -1,3 +1,7 @@
+"""Compatible import.
+"""
+
+from __future__ import unicode_literals
 import operator
 import string
 import sys
@@ -9,20 +13,14 @@ if _PY3:
     iteritems = operator.methodcaller("items")
     itervalues = operator.methodcaller("values")
     from urllib.parse import urlparse, ParseResult
-    from configparser import RawConfigParser
+    from configparser import ConfigParser
     binary_stdin = sys.stdin.buffer
     binary_stdout = sys.stdout.buffer
-    ascii_lowercase_unicode = string.ascii_lowercase
-    ascii_uppercase_unicode = string.ascii_uppercase
-    digits_unicode = string.digits
 else:
     from StringIO import StringIO
     iteritems = operator.methodcaller("iteritems")
     itervalues = operator.methodcaller("itervalues")
     from urlparse import urlparse, ParseResult
-    from ConfigParser import RawConfigParser
+    from ConfigParser import ConfigParser
     binary_stdin = sys.stdin
     binary_stdout = sys.stdout
-    ascii_lowercase_unicode = unicode(string.ascii_lowercase)
-    ascii_uppercase_unicode = unicode(string.ascii_uppercase)
-    digits_unicode = unicode(string.digits)
