@@ -1,3 +1,8 @@
+"""Utilitis for parsing URL and pattern.
+"""
+
+from __future__ import unicode_literals
+
 import hashlib
 from collections import namedtuple
 
@@ -25,29 +30,10 @@ class URLMeta(namedtuple('URLMeta', 'path_depth query_keys has_fragment')):
     """
     __slots__ = ()
 
-    def __hash__(self):
-        return hash(str(self))
-
-    def __str__(self):
-        s = StringIO()
-        s.write(str(self.path_depth))
-        if self.query_keys:
-            s.write(Symbols.QUESTION)
-            s.write(Symbols.AMPERSAND.join(self.query_keys))
-        if self.has_fragment:
-            s.write(Symbols.NUMBER)
-        s.seek(0)
-        return s.read()
-
     @property
     def depth(self):
         return self.path_depth + len(self.query_keys) + (1 if self.has_fragment else 0)
 
-    def __eq__(self, o):
-        if not isinstance(o, URLMeta):
-            return False
-        return hash(o) == hash(self)
-
 
 def specify_rule(rule, num):
     """Specify the format of the rule.
@@ -80,19 +66,19 @@ def specify_rule(rule, num):
     """
 
     if num == 1:
-        return u'[%s]' % rule
+        return '[%s]' % rule
     elif num < 0:
-        return u'[%s]+' % rule
+        return '[%s]+' % rule
     elif num > 1:
-        return u'[%s]{%d}' % (rule, num)
+        return '[%s]{%d}' % (rule, num)
     else:
         raise ValueError('Invalid num %s' % str(num))
 
 
 def wildcard_rule(rule):
-    """Specify the wildcard format of the rule.
+    """The wildcard format of the rule.
 
-    Shotcut of specify_rule(rule, -1).
+    Shortcut of specify_rule(rule, -1).
 
     Args:
         rule (str): The raw rule string to be secified.
@@ -225,7 +211,7 @@ def filter_useless(objs):
 
 
 def parse_query_string(query_string):
-    """Parse query string into keys and values
+    """Parse query string into keys and values.
 
     Args:
         query_string (str): The string to be parsed.
@@ -241,8 +227,8 @@ def parse_query_string(query_string):
     elif query_string == Symbols.EMPTY:
         return BLANK_TUPLE, BLANK_TUPLE
     elif query_string.endswith(Symbols.AMPERSAND):
-        raise IrregularURLException("Ends with '&' is invalid")
-    kv_type = True  # qkey True, qvalue False
+        raise IrregularURLException("Invalid '&' pos")
+    kv_type = True  # query_key True, query_value False
     last_c = None
     kv_buf = {True: StringIO(), False: StringIO()}
     kv_list = {True: [], False: []}
@@ -256,7 +242,7 @@ def parse_query_string(query_string):
             kv_type = not kv_type
         elif i == Symbols.AMPERSAND:
             if last_c is None or last_c == Symbols.AMPERSAND:
-                raise IrregularURLException("Series '&' is invalid")
+                raise IrregularURLException("Invalid '&' pos")
             s = kv_buf[kv_type]
             s.seek(0)
             kv_list[kv_type].append(s.read())
@@ -276,7 +262,7 @@ def parse_query_string(query_string):
     if kv_type:  # treat as value-less
         kv_list[False].append(Symbols.EMPTY)
 
-    # only one query without value, treat as key-less
+    # Only one query without value, treat as key-less.
     if len(kv_list[True]) == 1 and not kv_list[True][0].endswith(Symbols.EQUALS):
         kv_list[False][0], kv_list[True][0] = kv_list[True][0], kv_list[False][0]
     return tuple(kv_list[True]), tuple(kv_list[False])
@@ -311,16 +297,16 @@ def mix(pieces, rules):
             t_mix = True
         else:
             if t_rules and t_mix:
-                mixed_pieces.append(u''.join(t_pieces))
-                mixed_rules.append(u''.join(sorted(set(t_rules))))
+                mixed_pieces.append(''.join(t_pieces))
+                mixed_rules.append(''.join(sorted(set(t_rules))))
                 t_pieces = []
                 t_rules = []
             t_mix = False
         t_pieces.append(piece)
         t_rules.append(rule)
     if t_mix:
-        mixed_pieces.append(u''.join(t_pieces))
-        mixed_rules.append(u''.join(sorted(set(t_rules))))
+        mixed_pieces.append(''.join(t_pieces))
+        mixed_rules.append(''.join(sorted(set(t_rules))))
     else:
         mixed_pieces.extend(t_pieces)
         mixed_rules.extend(t_rules)
@@ -360,14 +346,14 @@ def unpack(result, normalize_key=True):
 
 
 def pack(url_meta, objs):
-    """Combine the objects into string based on URLMeta.
+    """Pack into URL-like string.
 
     Args:
         url_meta (URLMeta): The URLMeta object.
-        objs (sequence): The objects to be combined.
+        objs (sequence): The objects to be packed.
 
     Returns:
-        str: The combined string.
+        str: The packed URL-like string.
     """
     s = StringIO()
     s.write(Symbols.SLASH)
@@ -380,11 +366,11 @@ def pack(url_meta, objs):
         kv = zip(url_meta.query_keys,
                  [str(p) for p in objs[url_meta.path_depth:idx]])
         s.write(Symbols.AMPERSAND.join(
-            [u''.join((str(k), str(v))) for k, v in kv]))
+            [''.join((str(k), str(v))) for k, v in kv]))
 
     if url_meta.has_fragment:
         s.write(Symbols.NUMBER)
-        s.write(u''.join([str(p) for p in objs[idx:]]))
+        s.write(''.join([str(p) for p in objs[idx:]]))
     s.seek(0)
     return s.read()
 
@@ -403,16 +389,17 @@ def analyze_url(url):
     return unpack(result, True)
 
 
-def fuzzy_join(objs):
+def fuzzy_join(objs, sep='/'):
     """Join the fuzzy_rule of the objects into one string.
 
     Args:
         objs (sequence): The objects each of which have fuzzy_rule property.
+        sep (str): Defaults to '/'. Seperator for joining.
 
     Returns:
         str: The joined fuzzy_rule string.
     """
-    return u'/'.join([p.fuzzy_rule for p in objs])
+    return sep.join([p.fuzzy_rule for p in objs])
 
 
 class ParsedPiece(object):
@@ -441,7 +428,7 @@ class ParsedPiece(object):
     @property
     def fuzzy_rule(self):
         if not self._fuzzy_rule:
-            self._fuzzy_rule = u''.join(sorted(set(self.rules)))
+            self._fuzzy_rule = ''.join(sorted(set(self.rules)))
         return self._fuzzy_rule
 
     @property
@@ -464,7 +451,6 @@ class ParsedPiece(object):
 
         Returns:
             int: The literal length of the piece.
-
         """
         if self._piece_length < 0:
             piece = self.piece
@@ -499,7 +485,7 @@ class ParsedPiece(object):
     @property
     def piece(self):
         if self._piece is None:
-            self._piece = u''.join(self._pieces)
+            self._piece = ''.join(self._pieces)
         return self._piece
 
     def __str__(self):
@@ -512,20 +498,34 @@ EMPTY_PARSED_PIECE = ParsedPiece(EMPTY_TUPLE, EMPTY_TUPLE)
 
 
 class PieceParser(object):
-    """Parser to parse the piece of the URL.
+    """Parser to parse the URL piece.
 
     Used it to generate ParsedPiece object from the piece of URL.
     Not thread safe.
     """
+    __slots__ = ('_rules', '_pieces')
 
     def __init__(self):
         self._reset()
 
     def _reset(self):
-        self._rule_list = []
-        self._piece_list = []
+        self._rules = []
+        self._pieces = []
 
     def parse(self, piece):
+        """Parse a string into small sub-pieces with rules.
+
+        The consecutive charactors in the same charactor space
+        will be joined into one sub-piece, the corresponding
+        rule(charactor space) can also be got.
+
+        Args:
+            piece (str): A string to be parsed.
+
+        Returns:
+            tuple: 2-tuple, (pieces, rules).
+        """
+
         self._reset()
         self._preprocess(piece)
         return self._create_parsed_piece()
@@ -533,23 +533,22 @@ class PieceParser(object):
     def _preprocess(self, piece):
         for c in piece:
             self._define(c)
-        for idx, buf in enumerate(self._piece_list):
+        for idx, buf in enumerate(self._pieces):
             buf.seek(0)
             letter = buf.read()
-            self._piece_list[idx] = self._normalize(
-                letter, self._rule_list[idx])
+            self._pieces[idx] = self._normalize(
+                letter, self._rules[idx])
 
     def _define(self, char):
-        last_rule = self._rule_list[-1] if self._rule_list else None
-        try:
-            rule = CHAR_RULE_DICT[char]
-        except KeyError:
-            raise InvalidCharException("Contain invalid char")
+        last_rule = self._rules[-1] if self._rules else None
+        if char not in CHAR_RULE_DICT:
+            raise InvalidCharException("Invalid char %r" % char)
+        rule = CHAR_RULE_DICT[char]
 
         if last_rule != rule:
-            self._piece_list.append(StringIO())
-            self._rule_list.append(rule)
-        self._piece_list[-1].write(char)
+            self._pieces.append(StringIO())
+            self._rules.append(rule)
+        self._pieces[-1].write(char)
 
     def _normalize(self, letter, rule):
         if rule in SIGN_RULE_SET:
@@ -557,11 +556,24 @@ class PieceParser(object):
         return letter
 
     def _create_parsed_piece(self):
-        return ParsedPiece(tuple(self._piece_list), tuple(self._rule_list))
+        return ParsedPiece(tuple(self._pieces), tuple(self._rules))
+
+
+def fuzzy_digest(url_meta, objs):
+    """Generate hex digest string from URLMeta and objects' fuzzy_rules.
+
+    Args:
+        url_meta (URLMeta): The URLMeta object.
+        objs (sequence): Each object hava fuzzy_rule property.
+
+    Returns:
+        str: Digest value as a string of hexadecimal digits.
+    """
+    return digest(url_meta, [obj.fuzzy_rule for obj in objs])
 
 
 def digest(url_meta, objs):
-    """Get hex digest string from the given URLMeta and objects.
+    """Generate hex digest string from URLMeta and objects.
 
     Args:
         url_meta (URLMeta): The URLMeta object.
@@ -608,13 +620,13 @@ def parse_url_pattern_string(url_pattern_string):
 
 
 def analyze_url_pattern_string(url_pattern_string):
-    """Parse a URL pattern string to URLMeta object and pattern string pieces.
+    """Parse a URL pattern string into URLMeta object and pattern string pieces.
 
     Args:
         url_pattern_string (str): The URL pattern string to be parsed.
 
     Returns:
-        tuple: A 2-tuple, (url_meta, pattern_string_pieces).
+        tuple: A 2-tuple, (url_meta, pattern_strings).
     """
     result = parse_url_pattern_string(url_pattern_string)
     return unpack(result, False)
@@ -664,17 +676,19 @@ def parse_pattern_string(pattern_string):
                     num_str = pattern_string[old_idx:idx]
                     if not num_str.isdigit():
                         raise InvalidPatternException(
-                            "Invalid num '%s'" % num_str)
+                            "Invalid num %r" % num_str)
 
                 elif pattern_string[idx + 1] == Symbols.PLUS:
                     idx += 1
             idx += 1
             pattern_unit_strings.append(pattern_string[idx_s:idx])
         else:
+            if c not in CHAR_RULE_DICT:
+                raise InvalidPatternException("Invaid char %r" % c)
             rule = CHAR_RULE_DICT[c]
             if rule not in DIGIT_AND_ASCII_RULE_SET:
                 raise InvalidPatternException(
-                    'Invalid pattern: %s' % pattern_string)
+                    'Invalid pattern')
             if last_rule is None:
                 s.write(c)
             else:
@@ -717,7 +731,7 @@ def parse_pattern_unit_string(pattern_unit_string):
             t = pattern_unit_string.rfind(Symbols.BRACES_L)
             num_str = pattern_unit_string[t + 1:-1]
             if not num_str.isdigit():
-                raise InvalidPatternException("Invalid num '%s'" % num_str)
+                raise InvalidPatternException("Invalid num %r" % num_str)
             num = int(num_str)
         elif pattern_unit_string[-1] == Symbols.PLUS:
             num = -1
@@ -736,8 +750,9 @@ def parse_pattern_unit_string(pattern_unit_string):
                 n = 1
             rule = p_str[idx:idx + n]
             if rule not in RULE_SET:
-                raise InvalidPatternException(
-                    "Invalid pattern unit: %s" % pattern_unit_string)
+                raise InvalidPatternException("Invalid rule %r" % rule)
             rules.add(rule)
             idx += n
+    if (num > 0 and len(rules) > num) or num == 0:
+        raise InvalidPatternException('Insufficient number')
     return rules, num
