@@ -1,9 +1,13 @@
+"""Cluster algrithm.
+"""
+from __future__ import unicode_literals
+
 from collections import Counter, OrderedDict, namedtuple
 
 from .compat import itervalues
 from .parse_utils import (EMPTY_PARSED_PIECE, URLMeta, specify_rule,
                           wildcard_rule)
-from .parsed_piece_view import BaseView, LastDotSplitFuzzyView, MixedView
+from .parsed_piece_view import LastDotSplitFuzzyView, MixedView, MultiView
 from .pattern import Pattern
 from .piece_pattern_node import (PiecePatternNode, build_from_parsed_pieces,
                                  build_from_piece_pattern_nodes)
@@ -11,7 +15,7 @@ from .utils import Bag, cached_property, dump_tree, pick
 
 
 class TBag(Bag):
-    __slots__ = ('_stats')
+    __slots__ = ('_stats',)
 
     def __init__(self):
         super(TBag, self).__init__()
@@ -46,10 +50,6 @@ class TBucket(TBag):
     def __contains__(self, key):
         return key in self._objs
 
-    def _pick(self):
-        for obj in itervalues(self._objs):
-            return obj
-
     def __iter__(self):
         return iter(itervalues(self._objs))
 
@@ -63,7 +63,7 @@ class PieceBag(TBag):
     The nodes should on the same branch of a tree at the same level.
     """
 
-    __slots__ = ('_p_nodes')
+    __slots__ = ('_p_nodes',)
 
     def __init__(self):
         super(PieceBag, self).__init__()
@@ -82,7 +82,7 @@ class PieceBag(TBag):
 
 
 class PieceBagBucket(TBucket):
-    __slots__ = ('_p_nodes')
+    __slots__ = ('_p_nodes',)
 
     def __init__(self):
         super(PieceBagBucket, self).__init__()
@@ -147,22 +147,36 @@ class ViewPieceBagBucket(PieceBagBucket):
         pattern = None
         bucket = ViewPieceBagBucket(self._url_meta)
         for nodes in dump_tree(root):
-            piece = u''.join([p.piece for p in nodes[1:]])
+            piece = ''.join([p.piece for p in nodes[1:]])
             view_piece_bag = self[piece]
             bucket.add(view_piece_bag, False)
             if pattern is None:
                 pattern = Pattern(
-                    u''.join([str(p.pattern) for p in nodes[1:]]))
+                    ''.join([str(p.pattern) for p in nodes[1:]]))
         return bucket, pattern
 
 
-def confused(total, part, threshold):
+def confused(total, max_part, threshold):
+    """Determine whether it is too complex to become a cluster.
+
+    If a data set have several(<threshold) sub parts, this method use the total
+    count of the data set, the count of the max sub set and min cluster threshold
+    to determine whether it is too complex to become a cluster.
+
+    Args:
+        total (int): Total count of a data set.
+        max_part (int): The max part of the data set.
+        threshold (int): The min threashold of a cluster.
+
+    Returns:
+        bool: Too complex return True, otherwise False.
+    """
     if total < threshold:
         return False
-    o_part = total - part
-    if part >= threshold and o_part >= threshold:
+    o_part = total - max_part
+    if max_part >= threshold and o_part >= threshold:
         return True
-    return abs(part - o_part) < threshold - 1
+    return abs(max_part - o_part) < threshold - 1
 
 
 class SeekResult(object):
@@ -260,7 +274,7 @@ class PiecePatternCluster(PatternCluster):
             self._processor.get_cluster(LengthPatternCluster).add(piece_bag)
             return
 
-        view = BaseView(parsed_piece)
+        view = MultiView(parsed_piece)
         p_cls = BasePatternCluster
         vl = len(view.parsed_pieces)
 
@@ -348,8 +362,9 @@ class LengthPatternCluster(PatternCluster):
         parsed_piece = length_bucket.pick().parsed_piece
         length = parsed_piece.piece_length
         pattern = Pattern(specify_rule(parsed_piece.fuzzy_rule, length))
+        length_bucket.set_pattern(pattern)
         if update_patterns:
-            length_bucket.set_pattern(pattern)
+            self._patterns.add(pattern)
 
 
 class MultiPatternCluster(PatternCluster):
@@ -648,7 +663,7 @@ def split_by_pattern(root):
     """
     tree_roots = {}
     for nodes in dump_tree(root):
-        pid = hash(u"/".join([str(p.pattern) for p in nodes]))
+        pid = hash("/".join([str(p.pattern) for p in nodes]))
         if pid not in tree_roots:
             tree_roots[pid] = PiecePatternNode((EMPTY_PARSED_PIECE, None))
         sub_root = tree_roots[pid]
@@ -708,9 +723,11 @@ def cluster(config, url_meta, root, **kwargs):
         **kwargs: Keyword arguments.
 
     Yields:
-        PiecePatternNode: The clusterd sub piece pattern tree root.
+        PiecePatternNode: The clustered sub piece pattern tree root.
 
     """
+    if root.count <= 0:
+        return
     if not process(config, url_meta, root, **kwargs):
         yield root
         return
